@@ -3,7 +3,7 @@
 #
 # Extended mkmf for Ruby-GNOME2 and Ruby/GLib based libraries.
 #
-# Copyright(C) 2003-2013 Ruby-GNOME2 Project.
+# Copyright(C) 2003-2015 Ruby-GNOME2 Project.
 #
 # This program is licenced under the same
 # license of Ruby-GNOME2.
@@ -30,7 +30,7 @@ end
 
 def enable_debug_build_flag(flags)
   if gcc?
-    debug_option_pattern = /(^|\s)?-g\d?(\s|$)?/
+    debug_option_pattern = /(^|\s)-g\d?(\s|$)/
     if debug_option_pattern =~ flags
       flags.gsub(debug_option_pattern, '\\1-g3\\2')
     else
@@ -109,10 +109,13 @@ if ENV['GTK_BASEPATH'] and /cygwin/ !~ RUBY_PLATFORM
   $CFLAGS += " -I#{include_path} "
 end
 
+def windows_platform?
+  /cygwin|mingw|mswin/ === RUBY_PLATFORM
+end
+
 def setup_windows(target_name, base_dir=nil)
   checking_for(checking_message("Windows")) do
-    case RUBY_PLATFORM
-    when /cygwin|mingw|mswin/
+    if windows_platform?
       import_library_name = "libruby-#{target_name}.a"
       $DLDFLAGS << " -Wl,--out-implib=#{import_library_name}"
       $cleanfiles << import_library_name
@@ -137,19 +140,22 @@ end
 
 def find_gem_spec(package)
   begin
-    require 'rubygems'
-    if Gem::Specification.respond_to?(:find_by_name)
-      Gem::Specification.find_by_name(package)
-    else
-      Gem.source_index.find_name(package).last
-    end
+    Gem::Specification.find_by_name(package)
   rescue LoadError
     nil
   end
 end
 
+def setup_homebrew_libffi
+  return unless package_platform == :homebrew
+
+  PKGConfig.add_path("/usr/local/opt/libffi/lib/pkgconfig")
+end
+
 #add_depend_package("glib2", "ext/glib2", "/...../ruby-gnome2")
 def add_depend_package(target_name, target_srcdir, top_srcdir, options={})
+  setup_homebrew_libffi if target_name == "gobject-introspection"
+
   gem_spec = find_gem_spec(target_name)
   if gem_spec
     target_source_dir = File.join(gem_spec.full_gem_path, "ext/#{target_name}")
@@ -189,8 +195,7 @@ def add_depend_package_path(target_name, target_source_dir, target_build_dir)
     $INCFLAGS = "-I#{target_source_dir} #{$INCFLAGS}"
   end
 
-  case RUBY_PLATFORM
-  when /cygwin|mingw|mswin/
+  if windows_platform?
     target_base_dir = Pathname.new(target_source_dir).parent.parent
     target_binary_base_dir = target_base_dir + "vendor" + "local"
     if target_binary_base_dir.exist?
@@ -204,8 +209,7 @@ def add_depend_package_path(target_name, target_source_dir, target_build_dir)
     $INCFLAGS = "-I#{target_build_dir} #{$INCFLAGS}"
   end
 
-  case RUBY_PLATFORM
-  when /cygwin|mingw|mswin/
+  if windows_platform?
     library_base_name = "ruby-#{target_name.gsub(/-/, '_')}"
     case RUBY_PLATFORM
     when /cygwin|mingw/
@@ -408,34 +412,6 @@ def make_version_header(app_name, pkgname, dir = "src")
       out.close
 end
 
-def check_ruby_func
-  #Other options
-  ruby_header = "ruby.h"
-  have_func("rb_define_alloc_func", ruby_header) # for ruby-1.8
-  have_func("rb_block_proc", ruby_header) # for ruby-1.8
-
-  STDOUT.print("checking for new allocation framework... ") # for ruby-1.7
-  if Object.respond_to? :allocate
-    STDOUT.print "yes\n"
-    $defs << "-DHAVE_OBJECT_ALLOCATE"
-  else
-    STDOUT.print "no\n"
-  end
-
-  STDOUT.print("checking for attribute assignment... ") # for ruby-1.7
-  STDOUT.flush
-  if defined? try_compile and try_compile <<SRC
-#include "ruby.h"
-#include "node.h"
-int node_attrasgn = (int)NODE_ATTRASGN;
-SRC
-    STDOUT.print "yes\n"
-    $defs << "-DHAVE_NODE_ATTRASGN"
-  else
-    STDOUT.print "no\n"
-  end
-end
-
 def add_obj(name)
   ensure_objs
   $objs << name unless $objs.index(name)
@@ -449,12 +425,10 @@ def glib_mkenums(prefix, files, g_type_prefix, include_files, options={})
 end
 
 def check_cairo(options={})
-  is_windows = (/mingw|cygwin|mswin/ === RUBY_PLATFORM)
-
   rcairo_source_dir = options[:rcairo_source_dir]
   if rcairo_source_dir.nil?
     suffix = nil
-    if is_windows
+    if windows_platform?
       case RUBY_PLATFORM
       when /\Ax86-mingw/
         suffix = "win32"
@@ -479,7 +453,7 @@ def check_cairo(options={})
   end
 
   unless rcairo_source_dir.nil?
-    if is_windows
+    if windows_platform?
       options = {}
       build_dir = "tmp/#{RUBY_PLATFORM}/cairo/#{RUBY_VERSION}"
       if File.exist?(File.join(rcairo_source_dir, build_dir))
@@ -503,6 +477,10 @@ def package_platform
     :redhat
   elsif File.exist?("/etc/SuSE-release")
     :suse
+  elsif File.exist?("/etc/altlinux-release")
+    :altlinux
+  elsif find_executable("pacman")
+    :arch
   elsif find_executable("brew")
     :homebrew
   elsif find_executable("port")
@@ -534,12 +512,14 @@ def install_missing_native_package(native_package_info)
   package_command_line = [package_name, *options].join(" ")
   need_super_user_priviledge = true
   case platform
-  when :debian
+  when :debian, :altlinux
     install_command = "apt-get install -V -y #{package_command_line}"
   when :fedora, :redhat
     install_command = "yum install -y #{package_command_line}"
   when :suse
     install_command = "zypper --non-interactive install #{package_command_line}"
+  when :arch
+    install_command = "pacman -S --noconfirm #{package_command_line}"
   when :homebrew
     need_super_user_priviledge = false
     install_command = "brew install #{package_command_line}"
@@ -611,6 +591,9 @@ def required_pkg_config_package(package_info, native_package_info=nil)
   else
     required_package_info = [package_info]
   end
+  if required_package_info.include?("gobject-introspection-1.0")
+    setup_homebrew_libffi
+  end
   return true if PKGConfig.have_package(*required_package_info)
 
   native_package_info ||= {}
@@ -629,8 +612,6 @@ end
 add_include_path.call("sitearchdir")
 add_include_path.call("vendorarchdir")
 
-check_ruby_func
-
 if /mingw/ =~ RUBY_PLATFORM
   $ruby.gsub!('\\', '/')
 end
